@@ -26,85 +26,32 @@
 #include "ggit-oid.h"
 #include "ggit-convert.h"
 #include "ggit-tree.h"
+#include "ggit-commit-parents.h"
 
-#define GGIT_COMMIT_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GGIT_TYPE_COMMIT, GgitCommitPrivate))
+/**
+ * GgitCommit:
+ *
+ * Represents a commit object.
+ */
 
-struct _GgitCommitParents
-{
-	GgitCommit *commit;
-	gint ref_count;
-};
-
-struct _GgitCommitPrivate
+typedef struct _GgitCommitPrivate
 {
 	gchar *message_utf8;
 	gchar *subject_utf8;
-};
+} GgitCommitPrivate;
 
-G_DEFINE_TYPE (GgitCommit, ggit_commit, GGIT_TYPE_OBJECT)
-
-G_DEFINE_BOXED_TYPE (GgitCommitParents,
-                     ggit_commit_parents,
-                     ggit_commit_parents_ref,
-                     ggit_commit_parents_unref)
-
-static GgitCommitParents *
-ggit_commit_parents_wrap (GgitCommit *commit)
-{
-	GgitCommitParents *ret;
-
-	ret = g_slice_new (GgitCommitParents);
-	ret->commit = g_object_ref (commit);
-	ret->ref_count = 1;
-
-	return ret;
-}
-
-/**
- * ggit_commit_parents_ref:
- * @parents: a #GgitCommitParents.
- *
- * Atomically increments the reference count of @parents by one.
- * This function is MT-safe and may be called from any thread.
- *
- * Returns: a #GgitCommitParents.
- **/
-GgitCommitParents *
-ggit_commit_parents_ref (GgitCommitParents *parents)
-{
-	g_return_val_if_fail (parents != NULL, NULL);
-
-	g_atomic_int_inc (&parents->ref_count);
-
-	return parents;
-}
-
-/**
- * ggit_commit_parents_unref:
- * @parents: a #GgitCommitParents.
- *
- * Atomically decrements the reference count of @parents by one.
- * If the reference count drops to 0, @parents is freed.
- **/
-void
-ggit_commit_parents_unref (GgitCommitParents *parents)
-{
-	g_return_if_fail (parents != NULL);
-
-	if (g_atomic_int_dec_and_test (&parents->ref_count))
-	{
-		g_clear_object (&parents->commit);
-		g_slice_free (GgitCommitParents, parents);
-	}
-}
+G_DEFINE_TYPE_WITH_PRIVATE (GgitCommit, ggit_commit, GGIT_TYPE_OBJECT)
 
 static void
 ggit_commit_finalize (GObject *object)
 {
 	GgitCommit *commit = GGIT_COMMIT (object);
+	GgitCommitPrivate *priv;
 
-	g_free (commit->priv->message_utf8);
-	g_free (commit->priv->subject_utf8);
+	priv = ggit_commit_get_instance_private (commit);
+
+	g_free (priv->message_utf8);
+	g_free (priv->subject_utf8);
 
 	G_OBJECT_CLASS (ggit_commit_parent_class)->finalize (object);
 }
@@ -115,14 +62,11 @@ ggit_commit_class_init (GgitCommitClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = ggit_commit_finalize;
-
-	g_type_class_add_private (object_class, sizeof (GgitCommitPrivate));
 }
 
 static void
 ggit_commit_init (GgitCommit *self)
 {
-	self->priv = GGIT_COMMIT_GET_PRIVATE (self);
 }
 
 GgitCommit *
@@ -181,12 +125,15 @@ ggit_commit_get_message_encoding (GgitCommit *commit)
 static void
 ensure_message_utf8 (GgitCommit *commit)
 {
+	GgitCommitPrivate *priv;
 	git_commit *c;
 	const gchar *msg;
 	const gchar *encoding;
 	const gchar *ptr;
 
-	if (commit->priv->message_utf8)
+	priv = ggit_commit_get_instance_private (commit);
+
+	if (priv->message_utf8)
 	{
 		return;
 	}
@@ -196,17 +143,17 @@ ensure_message_utf8 (GgitCommit *commit)
 
 	encoding = ggit_commit_get_message_encoding (commit);
 
-	commit->priv->message_utf8 = ggit_convert_utf8 (msg,
-	                                                -1,
-	                                                encoding);
+	priv->message_utf8 = ggit_convert_utf8 (msg,
+	                                        -1,
+	                                        encoding);
 
 	/* Extract the subject */
-	ptr = g_utf8_strchr (commit->priv->message_utf8, -1, '\n');
+	ptr = g_utf8_strchr (priv->message_utf8, -1, '\n');
 
 	if (ptr != NULL)
 	{
-		commit->priv->subject_utf8 = g_strndup (commit->priv->message_utf8,
-		                                        ptr - commit->priv->message_utf8);
+		priv->subject_utf8 = g_strndup (priv->message_utf8,
+		                                ptr - priv->message_utf8);
 	}
 }
 
@@ -222,11 +169,15 @@ ensure_message_utf8 (GgitCommit *commit)
 const gchar *
 ggit_commit_get_message (GgitCommit *commit)
 {
+	GgitCommitPrivate *priv;
+
 	g_return_val_if_fail (GGIT_IS_COMMIT (commit), NULL);
 
+	priv = ggit_commit_get_instance_private (commit);
+
 	ensure_message_utf8 (commit);
 
-	return commit->priv->message_utf8;
+	return priv->message_utf8;
 }
 
 /**
@@ -242,17 +193,21 @@ ggit_commit_get_message (GgitCommit *commit)
 const gchar *
 ggit_commit_get_subject (GgitCommit *commit)
 {
+	GgitCommitPrivate *priv;
+
 	g_return_val_if_fail (GGIT_IS_COMMIT (commit), NULL);
 
+	priv = ggit_commit_get_instance_private (commit);
+
 	ensure_message_utf8 (commit);
 
-	if (commit->priv->subject_utf8)
+	if (priv->subject_utf8)
 	{
-		return commit->priv->subject_utf8;
+		return priv->subject_utf8;
 	}
 	else
 	{
-		return commit->priv->message_utf8;
+		return priv->message_utf8;
 	}
 }
 
@@ -319,81 +274,7 @@ ggit_commit_get_parents (GgitCommit *commit)
 {
 	g_return_val_if_fail (GGIT_IS_COMMIT (commit), NULL);
 
-	return ggit_commit_parents_wrap (commit);
-}
-
-/**
- * ggit_commit_parents_size:
- * @parents: a #GgitCommitParents.
- *
- * Get the number of parents in the parents collection.
- *
- * Returns: the number of parents.
- *
- **/
-guint
-ggit_commit_parents_size (GgitCommitParents *parents)
-{
-	git_commit *c;
-
-	g_return_val_if_fail (parents != NULL, 0);
-
-	c = _ggit_native_get (parents->commit);
-	return (guint)git_commit_parentcount (c);
-}
-
-/**
- * ggit_commit_parents_get:
- * @parents: a #GgitCommitParents.
- * @idx: the parent index.
- *
- * Get the #GgitCommit of a parent.
- *
- * Returns: (transfer full): a #GgitCommit.
- *
- **/
-GgitCommit *
-ggit_commit_parents_get (GgitCommitParents *parents,
-                         guint              idx)
-{
-	git_commit *c;
-	git_commit *p;
-
-	g_return_val_if_fail (parents != NULL, NULL);
-
-	c = _ggit_native_get (parents->commit);
-
-	if (git_commit_parent (&p, c, idx) == GIT_OK)
-	{
-		return _ggit_commit_wrap (p, TRUE);
-	}
-
-	return NULL;
-}
-
-/**
- * ggit_commit_parents_get_id:
- * @parents: a #GgitCommitParents.
- * @idx: the parent index.
- *
- * Get the #GgitOId of a parent.
- *
- * Returns: (transfer full): a #GgitOId.
- *
- **/
-GgitOId *
-ggit_commit_parents_get_id (GgitCommitParents *parents,
-                            guint              idx)
-{
-	git_commit *c;
-	const git_oid *oid;
-
-	g_return_val_if_fail (parents != NULL, NULL);
-
-	c = _ggit_native_get (parents->commit);
-
-	oid = git_commit_parent_id (c, idx);
-	return _ggit_oid_wrap (oid);
+	return ggit_commit_parents_new (commit);
 }
 
 /**
